@@ -17,6 +17,7 @@ import org.postgresql.util.PSQLException;
 // import org.checkerframework.checker.nullness.qual.Nullable;
 import org.junit.Assert;
 
+import java.io.Closeable;
 import java.io.File;
 import java.io.FileInputStream;
 import java.io.IOException;
@@ -340,11 +341,14 @@ public class TestUtil {
           "user name is not specified. Please specify 'username' property via -D or build.properties");
     }
     props.setProperty("user", user);
-    String password = getPassword();
+
+    // Allow properties to override the password.
+    String password = props.getProperty("password");
     if (password == null) {
-      password = "";
+      password = getPassword() != null ? getPassword() : "";
     }
     props.setProperty("password", password);
+
     String sslPassword = getSslPassword();
     if (sslPassword != null) {
       PGProperty.SSL_PASSWORD.set(props, sslPassword);
@@ -363,8 +367,10 @@ public class TestUtil {
     String hostport = props.getProperty(SERVER_HOST_PORT_PROP, getServer() + ":" + getPort());
     String database = props.getProperty(DATABASE_PROP, getDatabase());
 
-    // Set GSSEncMode for tests
-    PGProperty.GSS_ENC_MODE.set(props,getGSSEncMode().value);
+    // Set GSSEncMode for tests only in the case the property is already missing
+    if (PGProperty.GSS_ENC_MODE.getSetString(props) == null) {
+      PGProperty.GSS_ENC_MODE.set(props, getGSSEncMode().value);
+    }
 
     return DriverManager.getConnection(getURL(hostport, database), props);
   }
@@ -400,18 +406,7 @@ public class TestUtil {
    * Helper - drops a schema
    */
   public static void dropSchema(Connection con, String schema) throws SQLException {
-    Statement stmt = con.createStatement();
-    try {
-      if (con.getAutoCommit()) {
-        // Not in a transaction so ignore error for missing object
-        stmt.executeUpdate("DROP SCHEMA IF EXISTS " + schema + " CASCADE");
-      } else {
-        // In a transaction so do not ignore errors for missing object
-        stmt.executeUpdate("DROP SCHEMA " + schema + " CASCADE");
-      }
-    } finally {
-      closeQuietly(stmt);
-    }
+    dropObject(con, "SCHEMA", schema);
   }
 
   /*
@@ -439,7 +434,6 @@ public class TestUtil {
    * @param table String
    * @param columns String
    */
-
   public static void createTempTable(Connection con, String table, String columns)
       throws SQLException {
     Statement st = con.createStatement();
@@ -474,6 +468,24 @@ public class TestUtil {
     }
   }
 
+  /*
+   * Helper - creates a view
+   */
+  public static void createView(Connection con, String viewName, String query)
+      throws SQLException {
+    Statement st = con.createStatement();
+    try {
+      // Drop the view
+      dropView(con, viewName);
+
+      String sql = "CREATE VIEW " + viewName + " AS " + query;
+
+      st.executeUpdate(sql);
+    } finally {
+      closeQuietly(st);
+    }
+  }
+
   /**
    * Helper creates an enum type.
    *
@@ -481,7 +493,6 @@ public class TestUtil {
    * @param name String
    * @param values String
    */
-
   public static void createEnumType(Connection con, String name, String values)
       throws SQLException {
     Statement st = con.createStatement();
@@ -531,22 +542,11 @@ public class TestUtil {
    * Drops a domain.
    *
    * @param con Connection
-   * @param name String
+   * @param domain String
    */
-  public static void dropDomain(Connection con, String name)
+  public static void dropDomain(Connection con, String domain)
       throws SQLException {
-    Statement stmt = con.createStatement();
-    try {
-      if (con.getAutoCommit()) {
-        // Not in a transaction so ignore error for missing object
-        stmt.executeUpdate("DROP DOMAIN IF EXISTS " + name + " CASCADE");
-      } else {
-        // In a transaction so do not ignore errors for missing object
-        stmt.executeUpdate("DROP DOMAIN " + name + " CASCADE");
-      }
-    } finally {
-      closeQuietly(stmt);
-    }
+    dropObject(con, "DOMAIN", domain);
   }
 
   /**
@@ -572,50 +572,39 @@ public class TestUtil {
    * drop a sequence because older versions don't have dependency information for serials
    */
   public static void dropSequence(Connection con, String sequence) throws SQLException {
-    Statement stmt = con.createStatement();
-    try {
-      if (con.getAutoCommit()) {
-        // Not in a transaction so ignore error for missing object
-        stmt.executeUpdate("DROP SEQUENCE IF EXISTS " + sequence + " CASCADE");
-      } else {
-        // In a transaction so do not ignore errors for missing object
-        stmt.executeUpdate("DROP SEQUENCE " + sequence + " CASCADE");
-      }
-    } finally {
-      closeQuietly(stmt);
-    }
+    dropObject(con, "SEQUENCE", sequence);
   }
 
   /*
    * Helper - drops a table
    */
   public static void dropTable(Connection con, String table) throws SQLException {
-    Statement stmt = con.createStatement();
-    try {
-      if (con.getAutoCommit()) {
-        // Not in a transaction so ignore error for missing object
-        stmt.executeUpdate("DROP TABLE IF EXISTS " + table + " CASCADE ");
-      } else {
-        // In a transaction so do not ignore errors for missing object
-        stmt.executeUpdate("DROP TABLE " + table + " CASCADE ");
-      }
-    } finally {
-      closeQuietly(stmt);
-    }
+    dropObject(con, "TABLE", table);
+  }
+
+  /*
+   * Helper - drops a view
+   */
+  public static void dropView(Connection con, String view) throws SQLException {
+    dropObject(con, "VIEW", view);
   }
 
   /*
    * Helper - drops a type
    */
   public static void dropType(Connection con, String type) throws SQLException {
+    dropObject(con, "TYPE", type);
+  }
+
+  private static void dropObject(Connection con, String type, String name) throws SQLException {
     Statement stmt = con.createStatement();
     try {
       if (con.getAutoCommit()) {
         // Not in a transaction so ignore error for missing object
-        stmt.executeUpdate("DROP TYPE IF EXISTS " + type + " CASCADE");
+        stmt.executeUpdate("DROP " + type + " IF EXISTS " + name + " CASCADE");
       } else {
         // In a transaction so do not ignore errors for missing object
-        stmt.executeUpdate("DROP TYPE " + type + " CASCADE");
+        stmt.executeUpdate("DROP " + type + " " + name + " CASCADE");
       }
     } finally {
       closeQuietly(stmt);
@@ -825,6 +814,18 @@ public class TestUtil {
     return returnValue;
   }
 
+  /**
+   * Close a resource and ignore any errors during closing.
+   */
+  public static void closeQuietly(/* @Nullable */ Closeable resource) {
+    if (resource != null) {
+      try {
+        resource.close();
+      } catch (Exception ignore) {
+      }
+    }
+  }
+
   /**
    * Close a Connection and ignore any errors during closing.
    */
